import 'package:rxdart/rxdart.dart';

Map<T, List<S>> sortMapByKeys<S, T>(Map<T, List<S>> unsortedMap) {
  final _keys = unsortedMap.keys.toList();
  _keys.sort();
  var map = <T, List<S>>{};
  for (var key in _keys) {
    map.putIfAbsent(key, () => unsortedMap[key]);
  }
  return map;
}

Map<T, List<S>> filterMap<S, T>(
  Map<T, List<S>> unfilteredMap,
  bool Function(S) filter,
) {
  final _keys = unfilteredMap.keys.toList();
  _keys.sort();
  var map = <T, List<S>>{};
  for (var key in _keys) {
    final _list = unfilteredMap[key].where(filter).toList();
    if (_list.isNotEmpty) {
      map.putIfAbsent(key, () => _list);
    }
  }
  return map;
}

Map<String, dynamic> filterOutNulls(Map<String, dynamic> parameters) {
  final Map<String, dynamic> filtered = <String, dynamic>{};
  parameters.forEach((String key, dynamic value) {
    if (value != null) {
      filtered[key] = value;
    }
  });
  return filtered;
}

Stream<List<T>> mergeStreamsList<T>(List<Stream<List<T>>> streams,
        {int Function(T, T) sorter}) =>
    Observable.combineLatest<List<T>, List<T>>(streams, (list) {
      final _combined = flattenList(list);
      if (sorter != null) _combined.sort(sorter);
      return _combined.toList();
    });

Future<List<T>> mergeFuturesList<T>(List<Future<List<T>>> futures,
        {int Function(T, T) sorter}) =>
    Future.wait(futures).then((list) {
      final _combined = flattenList(list);
      if (sorter != null) _combined.sort(sorter);
      return _combined.toList();
    });

List<T> flattenList<T>(List<List<T>> list) {
  final flattened = list.reduce((l1, l2) {
    for (var item in l2) {
      if (!l1.contains(item)) {
        l1.add(item);
      }
    }

    return l1;
  });

  return flattened;
}
